import { useEffect, useState, useRef } from 'react'

/**
 * @date 2021/2/23
 * 
 * @description 同步useState 
 * 
 * @param { T } params 参数 
 * 
 * @return { Array }
 * 
 * @example 
 *  
 *  const [name , setName] = useStateSync<string>('怡宝')
 *  
 *  useEffect( () => {
 *      setName('农夫山泉', (res) => {
 *         console.log(res)  // 农夫山泉
 *      })
 *  }, [])
 * 
 */


export const useStateSync = <T>(params: T): Array<any> => {

    const cbRef = useRef((_params: T) => {})

    const [ state, setState ] = useState<T>(params)

    useEffect( () => {
        cbRef.current &&  cbRef.current(state)
    }, [state] )

 
    return [state,  (val: T, callback: (_params: T) => void): void => {
        cbRef.current = callback;
        setState(val);
    }]
}

 



export const useInterval = (callback, delay) => {
  const savedCallback = useRef();

  // Remember the latest callback.
  useEffect(() => {
    savedCallback.current = callback;
  });

  // Set up the interval.
  useEffect(() => {
    function tick() {
      savedCallback.current();
    }
    if (delay !== null) {
      let id = setInterval(tick, delay);
      return () => clearInterval(id);
    }
  }, [delay]);
}