import React, { useState, useEffect} from 'react'

import { View, Image } from '@tarojs/components'

import Taro from '@tarojs/taro'

import { isFirstState, getPointCard } from '../../api/Home'

import './index.scss'

type pointCardType =  {
    id: string,
    lock: boolean,
    label: any,
    title: string,
    pointState: number,
    [key: string]: any
}

type LockModal = {
    closeLockModal: () => void
}

const entranceList = [
    {
        id: 0,
        x: '51.47vw',
        y: '79.89vh'
    },
    {
        id: 1,
        x: '39.2vw',
        y: '66.26vh'
    },
    {
        id: 2,
        x: '71.2vw',
        y: '53.75vh'
    },
    {
        id: 3,
        x: '8.53vw',
        y: '50.62vh'
    },
    {
        id: 4,
        x: '45.33vw',
        y: '41.75vh'
    },
    {
        id: 5,
        x: '73.87vw',
        y: '28.57vh'
    },
    {
        id: 6,
        x: '37.33vw',
        y: '20.69vh'
    },
    {
        id: 7,
        x: '16vw',
        y: '7.27vh'
    }
]


const LockModal: React.FC<LockModal> = ({closeLockModal}) => {
    return (
        <React.Fragment>
        <View className='rule-pop'>
                <View className='container'>
                    <Image className='lock-bg' mode='aspectFill' src={require('../../assets/Entrance/lockBg.png')} ></Image>
                    <Image className='lock-btn' onClick={() => closeLockModal()} src={require('../../assets/Entrance/lockbtn.png')} ></Image>
                </View>
            </View>
       </React.Fragment>
    )
}

const Enteance: React.FC = () => {

    const [visible, setVisible] = useState<boolean>(false)

    const [ firstSate, setFirstState ] = useState<boolean>(false)

    const [ pointCard, setPointCard ] = useState<pointCardType[]>([])

    const [ curSite, setCurSite ] = useState<number>(0)

    useEffect( () => {
        IsFirstState()
    }, [])

    Taro.useDidShow ( () => {
        GetPointCard()
        Taro.removeStorageSync('cardIdList');
    })
  
    // 是否第一次闯关
    const IsFirstState = async () => {
       const data = await isFirstState()
       console.log(data);
       setFirstState(data)
    }

    // 获取关卡

    const GetPointCard = async () => {
        const data = await getPointCard()
      
        const assginData = data.map( item => {
            return {
                ...entranceList[item.pointState - 1],
                ...item,
            }
        })
        
        setPointCard(assginData)
        setCurSite( assginData.findIndex(item => !item.lock) - 1)
    }


    // 去我的页面
    const goMypage = () => {
        Taro.navigateTo({
             url: '/pages/My/index'
        });
    }

    // 开始闯关
    const start = (record: pointCardType) => {
        if (record.lock) {
            Taro.navigateTo({
                url: `/pages/TopicPage/index?pointState=${record.pointState}`
           });
        } else {
            setVisible(true)
        }
    }


    return (
        <View className='Enteance'>
            <View className='game-box'>
                <Image className='my' onClick={() => goMypage()} src={require('../../assets/Entrance/my.png')} />
            </View>

            {/* 游戏入口 */}

            {
                pointCard?.map( (item, index) => (
                    <View 
                      key={item.id} 
                      className='enteance-item' 
                      style={{top: item.y, left: item.x, position: 'absolute'}}
                      onClick={() => start(item)}
                    >
                        <Image
                          className='enteance-item-img' 
                          src={require(`../../assets/Entrance/${item.lock ? '' : 'g'}${item.pointState}.png`)}
                        />

                        <Image 
                          className='star' 
                          src={typeof item.starNum === 'number' ? require(`../../assets/Entrance/star${item.starNum}.png`) : ''}
                        />

                        {
                             curSite === index ? <Image 
                               className='step' 
                               style={{ left: item.pointState - 1 === 3 ? 64 : -55}} 
                               src={require(`../../assets/Entrance/${item.pointState - 1 === 3 ? 'setp-4' : 'step'}.png`)}
                             /> : null
                        }
                        
                    </View>
                ))
            }

            {/* 第一次进入的活动规则 */}
            {
               firstSate && <View className='rule-pop'>
                    <View className='container'>
                        <Image className='rule-img' src={require('../../assets/Entrance/first-tip.png')} ></Image>
                        <Image className='go-btn' onClick={() => setFirstState(false)} src={require('../../assets/Entrance/go.png')} ></Image>
                    </View>
                </View>
            }


            {/* 未解锁弹窗 */}
            { visible ? <LockModal closeLockModal={() => setVisible(false)} /> : null }
        </View>
    )
}

export default Enteance