import React, {useState, useEffect} from 'react'

import { View, Image, Text } from '@tarojs/components'

import Taro from '@tarojs/taro'

import { AtIcon } from 'taro-ui'

import './index.scss'

import { getPointCard } from '../../api/Home'

import { getPointHisory, getRank} from '../../api/My'

import Tip from '../../utils/tip'

import NavBar from '../../components/NavBar/index'



type RankListProps = {
    rankList: any,
    history: Record<string, any>[]
}



const transferTime = (_timeStamp: number) => {
    return _timeStamp < 10 ? '0' + _timeStamp : _timeStamp
}

const RankList: React.FC<RankListProps> = ({rankList, history}) => {


    // 当前选中的Menu下标
    const [MenuIndex, setMenuIndex] = useState(0)

    useEffect( () => {

    }, [MenuIndex])


    // 渲染排名等级
    const RenderThree = (rank: number) => {
        console.log();
        
        if (rank < 4) {
            return <Image  style={{width: 26, height: 29}} src={require(`../../assets/My/rank${rank}.png`)} />
        } else {
            return rank
        }
    }
    

    // 查看报告
    const lookReport = (item: any) => {

        const encodeId = encodeURIComponent( JSON.stringify([{id: item.id}]))

        console.log(encodeId, 'encodeId');
        
        Taro.navigateTo({
             url: `/pages/Report/index?cardIdList=${encodeId}`
        });
    }

    // 渲染做题记录

    const RenderTopicRecord = () => {

        const _transferTime = (timeStamp) => {
            if (timeStamp < 60) {
                return `0:${transferTime(timeStamp)}`
            } else if ( timeStamp >= 60 && timeStamp < 120 ) {

                let cache = timeStamp - 60
                return `1:${transferTime(cache)}`
            } else if ( timeStamp >= 120 && timeStamp < 180) {
                let cache = timeStamp - 120
                return `2:${transferTime(cache)}`
            }
        }

        return (
            <View className='topic'>
                {
                    history.length === 0 ? 
                    <View className='rank no-record'>
                        <Image  src='http://res.training.luojigou.vip/FjNk6pyGhegX8dgBvcYV7lTXs4AM' />
                    </View>
                    :
                    <View className='rank'>
                        {
                            history.map( item => (
                                <View className='topic-item' key={item.id} onClick={() => lookReport(item)}>
                                    <View className='left'>
                                        <View className='date'>{item.createTime.split(' ')[0]}</View>
                                        <View className='info'>
                                            <Text>做题卡一张</Text>
                                            <Text>用时{_transferTime(item.takeTime)}</Text>
                                            <Text>正确率{item.scoreRate}%</Text>
                                        </View>
                                    </View>
                                    <View className='right'>
                                        <View className='look-btn'>
                                            查看报告
                                        </View>
                                    </View>
                                
                                </View>
                            ))
                        }
                    </View>
                }
            </View>
        )
    }


    // 渲染排行榜
    const RenderRank = () => {

        const { myRank, gameRanks} = rankList

        return (
            <React.Fragment>
                {
                    myRank.id && 
                            <View className='rank-item'>
                            <View className='rank-count'>
                                {RenderThree(myRank.myRank)}
                            </View>
                            <Image className='user-ava'  src={myRank.userImage || require('../../assets/My/ava.jpg')} />
                            <View className='user-name over1'>{myRank.nickName || '逻辑狗'}</View>
                            <View className='success-rate'>正确率： <Text>{myRank?.scoreRate?.split('.')[0] || 0}%</Text></View>
                            <View className='date'>用时： <Text>{myRank.takeTime || 0}s</Text></View>
                    </View>
                }
              
                {
                    gameRanks.length === 0 ? 
                    <View className='rank no-record'>
                        <Image  src='http://res.training.luojigou.vip/FjNk6pyGhegX8dgBvcYV7lTXs4AM' />
                    </View>
                    :
                    <View className='rank'>
                    {
                        gameRanks.map( (item, index) =>  (
                            <View className='rank-item' key={item.id}>
                                <View className='rank-count'>
                                    {RenderThree(index + 1)}
                                </View>
                                <Image className='user-ava'  src={item.userImage || require('../../assets/My/ava.jpg')} />
                                <View className='user-name over1'>{item.nickName || '逻辑狗'}</View>
                                <View className='success-rate'>正确率： <Text>{item.scoreRate.split('.')[0] || 0}%</Text></View>
                                <View className='date'>用时： <Text>{item.takeTime || 0}s</Text></View>
                            </View>
                            // {item.takeTime}
                        ))
                    }  
                    </View>
                }
            </React.Fragment>
        )
    }

    return (
        <View className='RankList'>
            <View className='self'>
                <View className='menu'>
                    <View className={[MenuIndex === 0 ? 'active' : ''].join('')} onClick={() => setMenuIndex(0)}> 排行榜 </View>
                    <View className={[MenuIndex === 1 ? 'active' : ''].join('')} onClick={() => setMenuIndex(1)}> 做题记录 </View>
                </View>
                <View className='line'></View>
            </View>

            {/* 排行榜 */}
            {
                MenuIndex === 0 ? RenderRank() : RenderTopicRecord()
            }

            
        </View>
    )
}

const My: React.FC = () => {

    // 星级记录
    const [pointCard, setPointCard] = useState<Record<string, any>[]>([])

    // 当前选择的题卡
    const [ curId, setCurId ] = useState<string>('') 

    // 游戏排名
    const [ rankList, setRankList] = useState<Record<string, any>>({})

    // 做题记录
    const [ history, setHistory ] = useState<Record<string, any>[]>([])

    // 个人信息

    const [userInfo, setUserInfo ] = useState<Record<string, any>>([])

    useEffect( () => {
        GetPointCard()
        setUserInfo(Taro.getStorageSync('userInfo'))
    }, [])

    useEffect( () => {
        curId && GetRank()
        curId && GetPointHisory()
    // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [curId])

    // 获取星级

    const GetPointCard = async () => {
        Tip.loading()
        const data = await getPointCard()
        Tip.loaded()
        console.log(data);
        setPointCard(data)
        // 找到第一个没开启的关卡， 然后默认去选择最后一个开启的关卡
        const index  =  data.findIndex(item => !item.lock)

        // 没找到说明全部解锁， 默认最后一关
        if (index === -1) {
            setCurId( data[data.length - 1].id )
        } else {
            setCurId(data[index - 1].id)
        }
        
        
    }

    // 获取排行榜
    const GetRank = async () => {
        
        const data = await getRank(curId)
        
        setRankList(data)
        
    }

    // 获取做题记录
    const GetPointHisory = async () => {
        const data = await getPointHisory(curId)
        setHistory(data)
    }
    
    // 切换当前题卡
    const changePointCard = (item: any) => {
        setCurId(item.id)
    }

    // 去活动领取
    const goPage = () => {
        Taro.navigateTo({
             url: '/pages/GetBag/index'
        });
    }


    const RenderTopicResult = (item: any) => {
        if (item.lock) {
            if (typeof item.starNum  === 'number') {
                return <Image className='star' src={require(`../../assets/My/star${item.starNum}.png`)} />
            } else {
                return null
            }
            
        } else {
            return (
                <View className='clock'>
                    <Image style={{width: 8, height: 9}} src={require('../../assets/My/clock.png')} />
                    <Text>未解锁</Text>
                </View>
            )
        }
    }

    return (
        <View className='My'>
            <NavBar title='' icon={<AtIcon value='chevron-left' size='24' color='#fff'></AtIcon>} />
            <View className='header'></View>
            <View className='user'>
                <Image className='user-ava' src={userInfo.avatarUrl} />
                <Text>{userInfo.nickName}</Text>
            </View>
            {/* 闯关进度 */}
            <View className='customs-step'>
                {
                     pointCard?.map( (item, index) => (
                        <View 
                          className='step' 
                          style={{
                              marginRight: [3, 7].includes(index)?  0 : 27,
                              background: item.lock ? '#28D626' : 'rgba(40, 214, 38, 0.4)'
                            }} 
                          key={index}
                          onClick={() => changePointCard(item)}
                        >
                            <Image style={{width: 21, height: 27}} src={require(`../../assets/My/${index + 1 }.png`)} />
                            {
                                RenderTopicResult(item)
                            }
                            {
                                curId === item.id ?  <Image className='flag' src={require('../../assets/My/flag.png')} /> : null
                            }
                           
                        </View>
                    ))
                }
            </View>
            {/* 卡片 */}
            <View className='card' >
                <Image className='card-btn' onClick={goPage} src={require('../../assets/My/card-btn.png')} />
            </View>
            {/* 排行榜 */}
            {  rankList.hasOwnProperty('myRank')  ? <RankList rankList={rankList} history={history} /> : null }
        </View>
    )
}

export default My