import React, { useEffect, useState} from 'react'

import { View, Image, Text, WebView } from '@tarojs/components'

import Taro from '@tarojs/taro'

import './index.scss'

import Tip from '../../utils/tip'
 
import { getReport } from '../../api/Report'

type RateProps = {
    allQuestionNum: number,
    trueQuestionNum: number,
    takeTime: number,
    date: string
}

const Rate: React.FC<RateProps> = ({allQuestionNum, takeTime, date, trueQuestionNum}) => {

    const rateFn = (): string => ( trueQuestionNum / allQuestionNum * 100 ).toFixed(2)

    const formatDate = (): string => date.split(' ')[0].replace(/\-/g, '.')


    const RenderItem = () => {
        if (String(trueQuestionNum).length === 1) {
            return (
                <View className='percent'>
                        <View className='correct'>
                            {trueQuestionNum}
                        </View>
                        <View className='line'>
                            /
                        </View>
                        <View className='total'>
                            {allQuestionNum}
                        </View>
                </View>
            )
        } else {
            return (
                <View className='overflow-percent percent'>
                    <View className='overflow-correct'>
                        {trueQuestionNum}
                    </View>
                    <View className='overflow-line'>
                        /
                    </View>
                    <View className='overflow-total'>
                        {allQuestionNum}
                    </View>
                </View>
            )
        }
        
    }
    

    return (
        <View className='Rate'>
            <View className='ratebg-box'>
                <Text> 答对{trueQuestionNum}题，共{allQuestionNum}题 </Text>
            </View>
            <Image className='cloud' mode='aspectFill'  src={require('../../assets/Report/cloud.png')} />
            <RenderItem />
            <Image mode='aspectFill' className='dog' src={require('../../assets/Report/dog.png')} />
            <View className='data'>
                <View>
                    <Text className='label'>总时间：</Text>
                    <Text className='value'>{takeTime}s</Text>
                </View>
                <View>
                    <Text className='label'>正确率：</Text>
                    <Text className='value'>{rateFn()}%</Text>
                </View>
                <View>
                    <Text className='label'>时间：</Text>
                    <Text className='value'>{formatDate()}</Text>
                </View>
            </View>
        </View>
    )
}

type CardProps = {
    list: Record<string, any>[]
}

const Card: React.FC<CardProps> = ({list}) => {

    const transfer = (label: string) => {
        switch (label) {
            case '1':
              return '一'  
            case '2':
              return '二'  
            case '3':
              return '三'  
            case '4':
              return '四'  
            case '5':
              return '五'  
            case '6':
              return '六'  
            case '7':
              return '七'  
            case '8':
              return '八'  
              
        }
    }

    return (
        <View className='Card'>
            {
                list.length && list.map ( item => (
                    <View className='card-item' key={item.id}>
                        <View className='label'>第{transfer(item.cardId)}关卡片</View>
                        <View className='desc'>
                            <View className='type'>逻辑思维</View>
                            <View className='date'>
                                <Image src={require('../../assets/Report/watch.png')} />
                                <Text>{item.takeTime}s</Text>
                            </View>
                        </View>
                        <View className='line'></View>
                        <View className='star'>
                        {
                            new Array(item.trueQuestionNum).fill(0).map( (_item, index) => (
                                <Image key={index} src={require('../../assets/Report/star.png')} />
                            ))
  
                        }
                        {
                            new Array(item.allQuestionNum - item.trueQuestionNum).fill(0).map( (_item, index) => (
                                <Image key={index} src={require('../../assets/Report/stargrey.png')} />
                            ))
                        }
                        </View>
                    </View>
                ))
            }
            
        </View>
    )
}

const PromoteCard: React.FC = () => {

    const goGetBagPage = () => {
        Taro.navigateTo({
             url: '/pages/GetBag/index'
        });
   }

    return (
        <View className='PromoteCard'>
            <View className='content'>
                <View className='label'>数理逻辑能力提升方案</View>
                <View className='desc'>卡片闯关游戏是提升我们能力哦</View>
            </View>
            <Image mode='aspectFill' src={require('../../assets/Report/btn.png')} onClick={goGetBagPage} />
        </View>
    )
}

type CollectProps = {
    goWebViewPage: () => void
}

const Collect: React.FC<CollectProps> = ({goWebViewPage}) => {
    return (
        <View className='Collect'>
            <View className='label-box'>
                <Image className='collect-img' src={require('../../assets/Report/collect.png')} />
                <View className='label'>参与E20数理逻辑能力</View>
            </View>
            
            <View className='desc-box'>
                <View className='desc'>中国关工委健体中心素质能力培养计划</View>
                <View className='btn' onClick={() => goWebViewPage()}>活动报名</View>
            </View> 
        </View>
    )
}


type FloatCardProps = {
    ids: string[]
} 

const FloatCard: React.FC<FloatCardProps> = ({ids}) => {

    const goMyPage = () => Taro.navigateTo({ url: '/pages/My/index'})

    const goAnalysisPage = () => {
        const encodeIds = encodeURIComponent( JSON.stringify(ids) )

        console.log(encodeIds, 'encodeIds');
        
        Taro.navigateTo({
            url: `/pages/TopicPage/Analysis?ids=${encodeIds}`
       })
    }

    return (
        <View className='FloatCard'>
            <View className='card left' onClick={goAnalysisPage}>题卡解析</View>
            <View className='card right' onClick={goMyPage}>查看排行榜</View>
        </View>
    )
}


type reportDataType = {
    allQuestionNum: number,
    takeTime: number,
    trueQuestionNum:  number,
    date: string,
    list: Record<string, any>[]
}

const Report: React.FC = () => {

    const [ cardIdList, setCardIdList ] = useState<string[]>([])

    const [ reportData, setReportData ] = useState<reportDataType>({} as reportDataType )

    useEffect( () => {
        
        const _cardIdList: string[] = JSON.parse(decodeURIComponent(Taro.getCurrentInstance().router?.params.cardIdList as string)).map( item => item.id ) 

        console.log(_cardIdList, '_cardIdList');
        
        setCardIdList( _cardIdList)

    }, [])


    useEffect( () => {
        cardIdList.length && GetReport()
    // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [ cardIdList])

    // 获取游戏报告
    const GetReport = async () => {
        Tip.loading()
        const data =  await getReport(cardIdList)
        Tip.loaded()
        console.log(data);
        setReportData(data)
    }

    // 打开webview
    const goWebViewPage = () => {
        Taro.navigateTo({
             url: '/pages/WebView/index'
        });
    }

    return (
        <View className='Report' >
            {
                reportData.date && <View>
                    <Rate
                      allQuestionNum={reportData.allQuestionNum} 
                      takeTime={reportData.takeTime} 
                      date={reportData.date}
                      trueQuestionNum={reportData.trueQuestionNum}
                    />
                    <Card list={reportData.list} />
                    <PromoteCard />
                    <Collect goWebViewPage={goWebViewPage} />
                    <FloatCard ids={cardIdList} /> 
              </View>
            }
        </View>
    )
}
export default Report

