/* eslint-disable @typescript-eslint/no-use-before-define */
import React, { useEffect, useState } from 'react'

import { View , Image, Text} from '@tarojs/components'

import Taro from '@tarojs/taro'

import './Analysis.scss'

import TimeCom from './component/TimeCom'

import BgPanel from './component/BgPanel'

import { questionAnalysis } from '../../api/QuestionCard'

import Tip from '../../utils/tip'

const dotList = [
    {
        id: 0,
        x:  '0vw',
        y: '56.11vh'
    },
    {
        id: 1,
        x:  '0vw',
        y: '44.03vh'
    },
    {
        id: 2,
        x:  '0vw',
        y: '32.85vh'
    },
    {
        id: 3,
        x:  '0vw',
        y: '21.86vh'
    },
    {
        id: 4,
        x:  '84.8vw',
        y: '0vh'
    },
    {
        id: 5,
        x:  '60.27vw',
        y: '0vh'
    },
    {
        id: 6,
        x:  '36.27vw',
        y: '0vh'
    },
    {
        id: 6,
        x:  '10.36vw',
        y: '0vh'
    },
]

type CorrectAnswerProps = {
    trueAnswer: string[]
}

const CorrectAnswer: React.FC<CorrectAnswerProps> = ({trueAnswer}) => {

    
    return (
        <View className='CorrectAnswer'>
            <View className='label'><Text>正确答案:</Text> <Text>从左往右-从上往下</Text> </View>
            <View className='answer-box'>
                {
                    trueAnswer.length && trueAnswer.map( item => (
                        <View 
                          className='answer-item' 
                          key={item} 
                          style={{backgroundColor: item}}
                        />
                    )) 
                }
                
            </View>
        </View>
    )
}

type CourseProps = {
    content: any,
    options: any,
    index: number,
    trueAnswer: string[],
    label: string
}

const Course: React.FC<CourseProps> = ({content, options, index, trueAnswer, label}) => {



    // 屏幕信息
    const [ screen, setScreen ]  = useState<Record<string, any>>({})

    useEffect( () =>{
        Taro.getSystemInfo({
            success: function (res) {
                console.log(res);
                setScreen(res)
            }
        })
    }, [])

    return (
        // 2.3vh
        <View className='Course' >
            <BgPanel label={label} />

            {/* 绿色背景 */}
            {/* <Image className='Topicbg'  src={require('../../assets/TopicPage/Topicbg.png')} /> */}

            {
                dotList.map( item => (
                    <Image 
                      key={item.id} 
                      className={['dot', item.id >= 4 ? 'rotate-dot' : ''].join(',')} 
                      style={{right: item.x, bottom: item.y}} 
                      src={require('../../assets/TopicPage/dot.png')}
                    />
                ))
            }

            {/* 左侧显示面板 */}
            <View className='show-panel'>
                {/* 展示图 */}
                {
                   content.length && content?.map( item => (
                       <View 
                         className='game-item' 
                         key={item.id}
                         style={{
                             width: item.pw * (screen.screenWidth / 375) ,
                             height: item.ph * (screen.screenHeight / 812) ,
                             top: item.py * (screen.screenHeight / 812),
                             left:  item.px *  (screen.screenWidth / 375), 
                         }}
                       >
                           {/* 圆 */}
                           {/*     width: item.iw ,
                                height: item.ih, */}
                           <View 
                             className='round'
                             style={{
                                background: item.color,
                                width: item.iw * (screen.screenWidth / 375) ,
                                height: item.ih * (screen.screenHeight / 812),
                                top: item.iy * (screen.screenHeight / 812),
                                left: item.ix  * (screen.screenWidth / 375)
                             }}
                           />

                           {/* 图片 */}
                           <Image
                             className={['show-image'].join(',')}
                             src={item.imgUrl}
                             mode='aspectFit'
                             style={{
                                 width:   item.w * (screen.screenWidth / 375)  ,
                                 height:  item.h * (screen.screenHeight / 812) ,
                                 top:  item.y * (screen.screenHeight / 812),
                                 left:  item.x *  (screen.screenWidth / 375) ,
                                 position: 'absolute'
                             }}
                           />
                       </View>
                   )) 
                }
                
            </View>
            {/* 右侧选择面板 */}
            <View className='opra-panel' >
                {
                    options?.map( item => (
                        <View className='answer-item' key={item.id} >
                            <View className='answer-round'>
                                <View  className='round' style={{background: item.color}} />
                                {
                                    item.ifTrue ?  
                                        <Image className='correct' mode='aspectFit' src={require('../../assets/TopicPage/correct.png')} /> 
                                        :
                                        <Image className='correct' mode='aspectFit' src={require('../../assets/TopicPage/error.png')} />
                                    
                                }  
                            </View>
                            <Image  mode='aspectFill' src={item.imgUrl} />
                        </View>
                    ))
                }
            </View>


            <CorrectAnswer trueAnswer={trueAnswer} />
        </View>
    )
}

const transferTime = (_timeStamp: number) => {
    return _timeStamp < 10 ? '0' + _timeStamp : _timeStamp
}

type AnalysisItemProps = {
    item: any,
    index: number
}

const AnalysisItem: React.FC<AnalysisItemProps> = ({item, index}) => {

    const [ time , setTime] = useState<string>('0:00')

    useEffect( () => {
        tranferTime()
    // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [])

    const tranferTime = () => {
        const { takeTime } = item
        if ( takeTime < 60 ) {
            setTime(`0:${transferTime(takeTime)}`)
        } else if ( takeTime >= 60 && takeTime < 120 ) {

            let cache = takeTime - 60

            setTime(`1:${transferTime(cache)}`)
        } else if ( takeTime >= 120 && takeTime < 180) {
            let cache = takeTime - 120
            setTime(`2:${transferTime(cache)}`)
        } 
    }

    return (
        <View className='Analysis'>
            
            {
                index === 0 ? <Image className='bg' src={require('../../assets/TopicPage/bg.png')} /> : null
            }
            

            <TimeCom time={time} />

            <Course 
              content={item.content} 
              options={item.options} 
              index={index} 
              trueAnswer={item.trueAnswer} 
              label={item.label}
            />

        </View>
    )
}


const AnalysisPage: React.FC = () => {
    const [ ids, setIds ] =  useState<string[]>([])

    const [ Analysis, setAnalysis ] = useState<Record<string, any>[]>([])

    

    useEffect( () => {

        setIds( JSON.parse( decodeURIComponent(Taro.getCurrentInstance().router?.params.ids as string) ) )

    }, [])

    useEffect( () => {

        ids.length && QuestionAnalysis()
    
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [ids])

    // 获取错题解析数据
    const QuestionAnalysis = async () => {
       Tip.loading()
       const data = await questionAnalysis(ids)
       Tip.loaded()
       console.log(data);
       setAnalysis(data)

       
    }

    return (
        <React.Fragment>
            {
                Analysis.length && Analysis.map( (item, index) => (
                    <AnalysisItem key={index} item={item} index={index} />
                ))
            }
        </React.Fragment>
    )
}

export default AnalysisPage